==> v1.0.0.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/cube/utils/CubeUtils.java <==
/**
 * 
 */
package org.springblade.cube.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Stream;

import org.springblade.cube.dto.VirtualColumnParam;
import org.springblade.cube.enums.DateFormatEnum;
import org.springblade.cube.enums.DbMysqlEnums;
import org.springblade.cube.enums.NumberFormatEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.model.FileStructure;
import org.springblade.report.enums.DsTypeEnum;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
public class CubeUtils {
	
	final static Pattern CHINESE_PATTERN = Pattern.compile("[\u4e00-\u9fa5]");
	//(?:--)|
	final static Pattern SQL_INJECT_PATTERN = Pattern.compile("(/\\*(?:.|[\\n\\r])*?\\*/)|(\\b(update|delete|drop|insert|alter|set|truncate|declare|exec|master|execute)\\b)", Pattern.CASE_INSENSITIVE);
	
	/**
	 * 智能识别Excel列的类型
	 * 
	 * @param excelColumnIndex - 第几列
	 * @param dataList         - 数据集列表(一般Excel前10行)
	 * @return
	 */
	public static ViewColumnTypeEnum resolveDataType(Integer excelColumnIndex, List<Map<Integer, String>> dataList) {
		// 迭代
		int numberIndex = 0, datetimeIndex = 0, mobileIndex = 0;
		for (Map<Integer, String> dataMap : dataList) {
			String dataValue = dataMap.get(excelColumnIndex);
			if(StrUtil.isBlank(dataValue) || isNumber(dataValue.trim())) {
				numberIndex++;
			}
			if(StrUtil.isBlank(dataValue) || isDateTime(dataValue.trim())) {
				datetimeIndex++;
			}
			if(StrUtil.isBlank(dataValue) || isMobile(dataValue.trim())) {
				mobileIndex++;
			}
		}
		if(mobileIndex == dataList.size()) {
			return ViewColumnTypeEnum.STRING;
		}else if(numberIndex == dataList.size()) {
			return ViewColumnTypeEnum.NUMBER;
		}else if(datetimeIndex == dataList.size()) {
			return ViewColumnTypeEnum.DATETIME;
		}
		return ViewColumnTypeEnum.STRING;
	}

	
	public static String buildInsertSql(String tableName, List<FileStructure> fileStructureList) {
		return buildInsertSql(tableName, fileStructureList.stream().toArray(FileStructure[]::new));
	}
	/**
	 * 创建插入SQL
	 * 
	 * @param fileStructureList
	 * @return
	 */
	public static String buildInsertSql(String tableName, FileStructure[] fileStructureArray) {
		// 字段
		StringBuilder preFieldSql = new StringBuilder("(");
		Stream.of(fileStructureArray).map(FileStructure::getColumnName).forEach(f -> preFieldSql.append(f).append(","));
		preFieldSql.deleteCharAt(preFieldSql.length() - 1).append(")");

		// 数值占位符
		StringBuilder preValueSql = new StringBuilder("(");
		Stream.of(fileStructureArray).forEach(f -> preValueSql.append("?,"));
		preValueSql.deleteCharAt(preValueSql.length() - 1).append(")");
		return String.format(CubeConstants.INSERT_INTO_SQL_FORMAT, tableName, preFieldSql.toString(),
				preValueSql.toString());
	}
	
	/**
	 * 格式化虚拟字段公式
	 * 
	 * @param columnFormula
	 * @param tableColumnMap
	 * @return
	 */
	public static VirtualColumnParam formatColumnFormula(String columnFormula, Map<String, CubeTableColumn> tableColumnMap) {
		VirtualColumnParam virtualColumnParam = new VirtualColumnParam();
		List<String> columnNames = new ArrayList<String>();
		Matcher matcher = CubeConstants.COLUMN_FORMULA_PATTERN.matcher(columnFormula);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(1, group.length() - 1);
			CubeTableColumn cubeTableColumn = tableColumnMap.get(key);
			if (cubeTableColumn == null) {
				throw new RuntimeException("未找到需要替换的key：" + key);
			}
			columnNames.add(cubeTableColumn.getColumnName());
			columnFormula = columnFormula.replace(group,
					cubeTableColumn.getTableAliasName() + "." + cubeTableColumn.getColumnName());
		}
		virtualColumnParam.setColumnNames(columnNames);
		virtualColumnParam.setColumnSql(columnFormula);
		return virtualColumnParam;
	}


	/**
	 * 连接百分号
	 * @param numberFormat
	 * @param columnSql
	 * @return
	 */
	public static String percentConcat(NumberFormatEnum numberFormat, DsTypeEnum dsType, String columnSql) {
		if (NumberFormatEnum.isPercentFormat(numberFormat)) {
			if(dsType == DsTypeEnum.oracle) { //Oracle特殊处理
				return MultiDsSelectUtils.invokeEnumMethodType(dsType, DbMysqlEnums.MethodType.JOINT_PERCENT.name(), columnSql, numberFormat.getOracleFmFormat());
			}else {
				return MultiDsSelectUtils.invokeEnumMethodType(dsType, DbMysqlEnums.MethodType.JOINT_PERCENT.name(), columnSql);
			}
		}
		return columnSql;
	}
	
	/**
	 * 转换数据类型
	 * @param dsType
	 * @param oldColumnType
	 * @param columnType
	 * @param columnSql
	 * @param dateFormat
	 * @return
	 */
	public static String convertDataType(DsTypeEnum dsType, ViewColumnTypeEnum oldColumnType, ViewColumnTypeEnum columnType, String columnSql, DateFormatEnum dateFormat) {
		//解析出转换类型 @20220308 只支持[1.字符串/数字日期转Datetime,2.日期转字符串]
		String convertType = parseConvertType(oldColumnType, columnType);
		if(oldColumnType == null || oldColumnType == columnType || convertType == null) {
			return columnSql;
		}
		String dateTimeFormat = "";
		if(dateFormat != null && (oldColumnType == ViewColumnTypeEnum.DATETIME || columnType == ViewColumnTypeEnum.DATETIME)) { //如果
			dateTimeFormat = MultiDsSelectUtils.getEnumDateTimeFormat(dsType, dateFormat.name());
		}
		return MultiDsSelectUtils.invokeEnumMethodType(dsType, convertType, columnSql, dateTimeFormat);
	}

	/**
	 * 格式化日期
	 * @param dateFormat
	 * @param dsType
	 * @param columnSql
	 * @return
	 */
	public static String formatTimeColumn(DateFormatEnum dateFormat, DsTypeEnum dsType, String columnSql) {
		//1.获取每个数据库的日期格式化 格式
		String dateTimeFormat = MultiDsSelectUtils.getEnumDateTimeFormat(dsType, dateFormat.name());
		if(StrUtil.isNotBlank(dateTimeFormat)) {
			//2.Date转Str函数
			return MultiDsSelectUtils.invokeEnumMethodType(dsType, DbMysqlEnums.MethodType.DATE_TO_STR.name(), columnSql, dateTimeFormat);
		}
		return columnSql;
	}
	
	/**
	 * 是否包含中文字符串
	 * @param str
	 * @return
	 */
	public static boolean isContainsChinese(String str) {
		return CHINESE_PATTERN.matcher(str).find();
	}
	
	/**
	 * 校验是否Sql注入
	 * @param sql
	 * @return
	 */
	public static boolean checkSqlInject(String sql) {
		return SQL_INJECT_PATTERN.matcher(sql).find();
	}
	
	/**
	 * 是否数字 支持格式:[1-带空格的纯数值/2-带,千位符的纯数值/3-带%的百分数/4-货币]
	 * 
	 * @param value
	 * @return
	 */
	static boolean isNumber(String value) {
		try {
			// 货币字符串替换掉¥/$/%,以及逗号
			value = value.replaceAll(CubeConstants.NUMBER_FORAMT_PATTERNS, "");
			Double doubleValue = new Double(value);
			//建表时，数字的最大字段长度30
			return doubleValue < Double.MAX_VALUE && value.length() <= 30;
		} catch (Exception e) {
		}
		return false;
	}
	
	/**
	 * 是否日期类型
	 * 
	 * @param value
	 * @return
	 */
	static boolean isDateTime(String value) {
		try {
			DateUtil.parse(value, CubeConstants.DATE_FORMAT_PATTERNS);
			return true;
		} catch (Exception e) {
		}
		return false;
	}
	
	/**
	 * 是否手机号
	 * @param value
	 * @return
	 */
	static boolean isMobile(String value) {
		return value.startsWith("1") && value.length() == 11;
	}
	
	/**
	 * 解析哪种数据转换
	 * @param oldColumnType
	 * @param columnType
	 * @return
	 */
	static String parseConvertType(ViewColumnTypeEnum oldColumnType, ViewColumnTypeEnum columnType) {
		if(oldColumnType == ViewColumnTypeEnum.STRING && columnType == ViewColumnTypeEnum.DATETIME) {
			return DbMysqlEnums.MethodType.STR_TO_DATE.name();
		}else if(oldColumnType == ViewColumnTypeEnum.NUMBER && columnType == ViewColumnTypeEnum.DATETIME) {
			return DbMysqlEnums.MethodType.NUMBER_TO_DATE.name();
		}else if(oldColumnType == ViewColumnTypeEnum.DATETIME && columnType == ViewColumnTypeEnum.STRING) {
			return DbMysqlEnums.MethodType.DATE_TO_STR.name();
		}
		return null;
	}
}

==> v1.1.1.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/cube/utils/CubeUtils.java <==
/**
 * 
 */
package org.springblade.cube.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.VirtualColumnParam;
import org.springblade.cube.enums.DateFormatEnum;
import org.springblade.cube.enums.DbMysqlEnums;
import org.springblade.cube.enums.NumberFormatEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.model.FileStructure;
import org.springblade.report.enums.DsTypeEnum;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
public class CubeUtils {
	
	final static Pattern CHINESE_PATTERN = Pattern.compile("[\u4e00-\u9fa5]");
	//(?:--)|
	final static Pattern SQL_INJECT_PATTERN = Pattern.compile("(/\\*(?:.|[\\n\\r])*?\\*/)|(\\b(update|delete|drop|insert|alter|set|truncate|declare|exec|master|execute)\\b)", Pattern.CASE_INSENSITIVE);
	//计算字段全局变量参数匹配
	final static Pattern VIRTUAL_COLUMN_PARAM_PATTERN = Pattern.compile("\\$\\{.*?\\}");
	
	/**
	 * 智能识别Excel列的类型
	 * 
	 * @param excelColumnIndex - 第几列
	 * @param dataList         - 数据集列表(一般Excel前10行)
	 * @return
	 */
	public static ViewColumnTypeEnum resolveDataType(Integer excelColumnIndex, List<Map<Integer, String>> dataList) {
		// 迭代
		int numberIndex = 0, datetimeIndex = 0, mobileIndex = 0;
		for (Map<Integer, String> dataMap : dataList) {
			String dataValue = dataMap.get(excelColumnIndex);
			if(StrUtil.isBlank(dataValue) || _isNumber(dataValue.trim())) {
				numberIndex++;
			}
			if(StrUtil.isBlank(dataValue) || _isDateTime(dataValue.trim())) {
				datetimeIndex++;
			}
			if(StrUtil.isBlank(dataValue) || _isMobile(dataValue.trim())) {
				mobileIndex++;
			}
		}
		if(mobileIndex == dataList.size()) {
			return ViewColumnTypeEnum.STRING;
		}else if(numberIndex == dataList.size()) {
			return ViewColumnTypeEnum.NUMBER;
		}else if(datetimeIndex == dataList.size()) {
			return ViewColumnTypeEnum.DATETIME;
		}
		return ViewColumnTypeEnum.STRING;
	}

	
	public static String buildInsertSql(String tableName, List<FileStructure> fileStructureList) {
		return buildInsertSql(tableName, fileStructureList.stream().toArray(FileStructure[]::new));
	}
	/**
	 * 创建插入SQL
	 * 
	 * @param fileStructureList
	 * @return
	 */
	public static String buildInsertSql(String tableName, FileStructure[] fileStructureArray) {
		// 字段
		StringBuilder preFieldSql = new StringBuilder("(");
		Stream.of(fileStructureArray).map(FileStructure::getColumnName).forEach(f -> preFieldSql.append(f).append(","));
		preFieldSql.deleteCharAt(preFieldSql.length() - 1).append(")");

		// 数值占位符
		StringBuilder preValueSql = new StringBuilder("(");
		Stream.of(fileStructureArray).forEach(f -> preValueSql.append("?,"));
		preValueSql.deleteCharAt(preValueSql.length() - 1).append(")");
		return String.format(CubeConstants.INSERT_INTO_SQL_FORMAT, tableName, preFieldSql.toString(),
				preValueSql.toString());
	}
	
	/**
	 * 格式化虚拟字段公式
	 * 
	 * @param columnFormula
	 * @param tableColumnMap
	 * @return
	 */
	public static VirtualColumnParam formatColumnFormula(String columnFormula, Map<String, CubeTableColumn> tableColumnMap) {
		VirtualColumnParam virtualColumnParam = new VirtualColumnParam();
		List<String> columnNames = new ArrayList<String>();
		Matcher matcher = CubeConstants.COLUMN_FORMULA_PATTERN.matcher(columnFormula);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(1, group.length() - 1);
			CubeTableColumn cubeTableColumn = tableColumnMap.get(key);
			if (cubeTableColumn == null) {
				throw new ServiceException("未找到需要替换的key：" + key);
			}
			columnNames.add(cubeTableColumn.getColumnName());
			columnFormula = columnFormula.replace(group, cubeTableColumn.getTableAliasName() + "." + cubeTableColumn.getColumnName());
		}
		virtualColumnParam.setColumnNames(columnNames);
		virtualColumnParam.setColumnSql(columnFormula);
		return virtualColumnParam;
	}

	/**
	 * 计算字段片段SQL替换全局变量
	 * 
	 * @param virtualSqlFragment
	 * @param extraParams
	 * @return
	 */
	public static String formatVirtualSqlFragment(String virtualSqlFragment, Map<String, List<Object>> extraParams) {
		Matcher matcher = VIRTUAL_COLUMN_PARAM_PATTERN.matcher(virtualSqlFragment);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(2, group.length() - 1);
			//.filter(item -> StrUtil.isNotBlank(item.toString().trim()))
			List<String> paramValList = Optional.<List<Object>>ofNullable(extraParams.get(key)).orElse(new ArrayList<>()).stream().map(Object::toString).collect(Collectors.toList());
			if (CollectionUtil.isEmpty(paramValList)) {
				throw new ServiceException("计算字段替换全局变量失败|未找到需要替换的key：" + key);
			}
			virtualSqlFragment = virtualSqlFragment.replace(group, String.join(",", paramValList));
		}
		return virtualSqlFragment;
	}

	/**
	 * 连接百分号
	 * @param numberFormat
	 * @param columnSql
	 * @return
	 */
	public static String percentConcat(NumberFormatEnum numberFormat, DsTypeEnum dsType, String columnSql) {
		if (NumberFormatEnum.isPercentFormat(numberFormat)) {
			if(dsType == DsTypeEnum.oracle) { //Oracle特殊处理
				return MultiDsSelectUtils.invokeEnumMethodType(dsType, DbMysqlEnums.MethodType.JOINT_PERCENT.name(), columnSql, numberFormat.getOracleFmFormat());
			}else {
				return MultiDsSelectUtils.invokeEnumMethodType(dsType, DbMysqlEnums.MethodType.JOINT_PERCENT.name(), columnSql);
			}
		}
		return columnSql;
	}
	
	/**
	 * 转换数据类型
	 * @param dsType
	 * @param oldColumnType
	 * @param columnType
	 * @param columnSql
	 * @param dateFormat
	 * @return
	 */
	public static String convertDataType(DsTypeEnum dsType, ViewColumnTypeEnum oldColumnType, ViewColumnTypeEnum columnType, String columnSql, DateFormatEnum dateFormat) {
		//解析出转换类型 @20220308 只支持[1.字符串/数字日期转Datetime,2.日期转字符串]
		String convertType = _parseConvertType(oldColumnType, columnType);
		if(oldColumnType == null || oldColumnType == columnType || convertType == null) {
			return columnSql;
		}
		String dateTimeFormat = "";
		if(dateFormat != null && (oldColumnType == ViewColumnTypeEnum.DATETIME || columnType == ViewColumnTypeEnum.DATETIME)) { //如果
			dateTimeFormat = MultiDsSelectUtils.getEnumDateTimeFormat(dsType, dateFormat.name());
		}
		return MultiDsSelectUtils.invokeEnumMethodType(dsType, convertType, columnSql, dateTimeFormat);
	}

	/**
	 * 格式化日期
	 * @param dateFormat
	 * @param dsType
	 * @param columnSql
	 * @return
	 */
	public static String formatTimeColumn(DateFormatEnum dateFormat, DsTypeEnum dsType, String columnSql) {
		//1.获取每个数据库的日期格式化 格式
		String dateTimeFormat = MultiDsSelectUtils.getEnumDateTimeFormat(dsType, dateFormat.name());
		if(StrUtil.isNotBlank(dateTimeFormat)) {
			//2.Date转Str函数
			return MultiDsSelectUtils.invokeEnumMethodType(dsType, DbMysqlEnums.MethodType.DATE_TO_STR.name(), columnSql, dateTimeFormat);
		}
		return columnSql;
	}
	
	/**
	 * 是否包含中文字符串
	 * @param str
	 * @return
	 */
	public static boolean isContainsChinese(String str) {
		return CHINESE_PATTERN.matcher(str).find();
	}
	
	/**
	 * 校验是否Sql注入
	 * @param sql
	 * @return
	 */
	public static boolean checkSqlInject(String sql) {
		return SQL_INJECT_PATTERN.matcher(sql).find();
	}
	
	/**
	 * 是否数字 支持格式:[1-带空格的纯数值/2-带,千位符的纯数值/3-带%的百分数/4-货币]
	 * 
	 * @param value
	 * @return
	 */
	static boolean _isNumber(String value) {
		try {
			// 货币字符串替换掉¥/$/%,以及逗号
			value = value.replaceAll(CubeConstants.NUMBER_FORAMT_PATTERNS, "");
			Double doubleValue = new Double(value);
			//建表时，数字的最大字段长度30
			return doubleValue < Double.MAX_VALUE && value.length() <= 30;
		} catch (Exception e) {
		}
		return false;
	}
	
	/**
	 * 是否日期类型
	 * 
	 * @param value
	 * @return
	 */
	static boolean _isDateTime(String value) {
		try {
			DateUtil.parse(value, CubeConstants.DATE_FORMAT_PATTERNS);
			return true;
		} catch (Exception e) {
		}
		return false;
	}
	
	/**
	 * 是否手机号
	 * @param value
	 * @return
	 */
	static boolean _isMobile(String value) {
		return value.startsWith("1") && value.length() == 11;
	}
	
	/**
	 * 解析哪种数据转换
	 * @param oldColumnType
	 * @param columnType
	 * @return
	 */
	static String _parseConvertType(ViewColumnTypeEnum oldColumnType, ViewColumnTypeEnum columnType) {
		if(oldColumnType == ViewColumnTypeEnum.STRING && columnType == ViewColumnTypeEnum.DATETIME) {
			return DbMysqlEnums.MethodType.STR_TO_DATE.name();
		}else if(oldColumnType == ViewColumnTypeEnum.NUMBER && columnType == ViewColumnTypeEnum.DATETIME) {
			return DbMysqlEnums.MethodType.NUMBER_TO_DATE.name();
		}else if(oldColumnType == ViewColumnTypeEnum.DATETIME && columnType == ViewColumnTypeEnum.STRING) {
			return DbMysqlEnums.MethodType.DATE_TO_STR.name();
		}
		return null;
	}
}
